@@ -13,18 +13,24 @@
 # under the License.
 #
 
+import acos_client.errors as acos_errors
 from oslo_config import cfg
 from oslo_log import log as logging
+from requests import exceptions as req_exceptions
 import six
 from taskflow import task
 from taskflow.types import failure
 
 from octavia.common import constants
-from octavia.common import utils
 from octavia.controller.worker import task_utils
 from octavia.network import base
 from octavia.network import data_models as n_data_models
 
+from a10_octavia.common import a10constants
+from a10_octavia.common import exceptions
+from a10_octavia.common import utils as a10_utils
+from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -40,7 +46,7 @@ class BaseNetworkTask(task.Task):
     @property
     def network_driver(self):
         if self._network_driver is None:
-            self._network_driver = utils.get_network_driver()
+            self._network_driver = a10_utils.get_network_driver()
         return self._network_driver
 
 
@@ -289,8 +295,9 @@ class HandleNetworkDeltas(BaseNetworkTask):
                                                        nic.network_id)
                 except base.NetworkNotFound:
                     LOG.debug("Network %d not found ", nic.network_id)
-                except Exception:
-                    LOG.exception("Unable to unplug network")
+                except Exception as e:
+                    LOG.exception("Unable to unplug network due to: %s", str(e))
+                    raise e
         return added_ports
 
     def revert(self, result, deltas, *args, **kwargs):
@@ -658,3 +665,144 @@ class ApplyQosAmphora(BaseNetworkTask):
         except Exception as e:
             LOG.error('Failed to remove QoS policy: %s from port: %s due '
                       'to error: %s', orig_qos_id, amp_data.vrrp_port_id, e)
+
+
+class HandleVRIDFloatingIP(BaseNetworkTask):
+    """Handle VRID floating IP configurations for member"""
+
+    def __init__(self, *arg, **kwargs):
+        self.fip_port = None
+        super(HandleVRIDFloatingIP, self).__init__(*arg, **kwargs)
+
+    @axapi_client_decorator
+    def execute(self, vthunder, member, vrid):
+        device_vrid_ip = None
+        if vrid:
+            device_vrid_ip = vrid.vrid_floating_ip
+
+        conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(member.project_id)
+        if conf_floating_ip:
+            subnet = self.network_driver.get_subnet(member.subnet_id)
+            subnet_ip, subnet_mask = a10_utils.get_net_info_from_cidr(subnet.cidr)
+            if conf_floating_ip.lower() == 'dhcp':
+                if not a10_utils.check_ip_in_subnet_range(device_vrid_ip, subnet_ip, subnet_mask):
+                    try:
+                        self.fip_port = self.network_driver.create_port(subnet.network_id,
+                                                                        member.subnet_id)
+                    except Exception as e:
+                        LOG.error("Failed to create neutron port for member: %s", member.id)
+                        raise e
+
+            else:
+                conf_floating_ip = a10_utils.get_patched_ip_address(conf_floating_ip, subnet.cidr)
+                if not a10_utils.check_ip_in_subnet_range(conf_floating_ip, subnet_ip, subnet_mask):
+                    msg = "Invalid VRID floating IP. IP out of subnet range: "
+                    msg += str(conf_floating_ip)
+                    raise exceptions.VRIDIPNotInSubentRangeError(msg)
+
+                if conf_floating_ip != device_vrid_ip:
+                    try:
+                        self.fip_port = self.network_driver.create_port(subnet.network_id,
+                                                                        member.subnet_id,
+                                                                        fixed_ip=conf_floating_ip)
+                    except Exception as e:
+                        LOG.error("Failed to create neutron port for member: %s with "
+                                  "floating IP %s", member.id, conf_floating_ip)
+                        raise e
+
+            if self.fip_port:
+                self.update_device_vrid_fip(self.fip_port.fixed_ips[0].ip_address, vthunder, vrid)
+
+        if vrid and vrid.vrid_port_id and (self.fip_port or not conf_floating_ip):
+            try:
+                self.network_driver.delete_port(vrid.vrid_port_id)
+            except Exception as e:
+                LOG.error("Failed to delete neutron port: %s for member %s on vrid %s",
+                          vrid.vrid_port_id, member.id, str(vrid))
+                raise e
+            if not conf_floating_ip:
+                try:
+                    self.axapi_client.vrrpa.delete(vrid.vrid)
+                except Exception as e:
+                    LOG.exceptions("Failed to delete vrid %s for member %s", str(vrid), member.id)
+                    raise e
+
+        return self.fip_port
+
+    @axapi_client_decorator
+    def revert(self, result, vthunder, member, vrid, *args, **kwargs):
+        if isinstance(result, failure.Failure):
+            LOG.exception("Unable to allocate & configure VRRP Floating IP Port")
+            return
+
+        if self.fip_port:
+            LOG.warning("Reverting VRRP floating IP delta task for vrid %s on member %s",
+                        str(vrid), member.id)
+            try:
+                self.network_driver.delete_port(self.fip_port.id)
+                if vrid:
+                    self.axapi_client.vrrpa.update(vrid.vrid, vrid.vrid_floating_ip)
+            except req_exceptions.ConnectionError:
+                LOG.exception("Failed to connect A10 Thunder device: %s", vthunder.ip_address)
+            except Exception as e:
+                LOG.exception("Failed to revert VRRP floating IP delta task for member: %s"
+                              " due to %s", member.id, str(e))
+
+    def update_device_vrid_fip(self, conf_floating_ip, vthunder, vrid):
+        vrid_value = CONF.a10_global.vrid
+        if vrid:
+            vrid_value = vrid.vrid
+        try:
+            if not vthunder.partition_name or vthunder.partition_name == 'shared':
+                self.axapi_client.vrrpa.update(vrid_value, floating_ip=conf_floating_ip)
+            else:
+                self.axapi_client.vrrpa.update(vrid_value, floating_ip=conf_floating_ip,
+                                               is_partition=True)
+        except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
+            LOG.exception("Failed to update VRRP floating IP %s for vrid: %s",
+                          conf_floating_ip, str(vrid_value))
+            raise e
+
+
+class DeleteMemberVRIDPort(BaseNetworkTask):
+    """Delete VRID Port if the last member associated with it is deleted
+    """
+    @axapi_client_decorator
+    def execute(self, vthunder, vrid, member_count):
+        if vrid and member_count == 1:
+            try:
+                self.network_driver.delete_port(vrid.vrid_port_id)
+                self.axapi_client.vrrpa.delete(vrid.vrid)
+                LOG.info("VRID port : %s deleted", vrid.vrid_port_id)
+                return True
+            except Exception as e:
+                LOG.exception("Failed to delete vrid port : %s", str(e))
+                raise e
+        return False
+
+
+class GetSubnetVLANIDParent(object):
+    """Get the Subnet VLAN_ID"""
+
+    def get_vlan_id(self, subnet_id):
+        network_id = self.network_driver.get_subnet(subnet_id).network_id
+        network = self.network_driver.get_network(network_id)
+        if network.provider_network_type != 'vlan':
+            raise
+        return network.provider_segmentation_id
+
+
+class GetVipSubnetVLANID(GetSubnetVLANIDParent, BaseNetworkTask):
+
+    default_provides = a10constants.VLAN_ID
+
+    def execute(self, loadbalancer):
+        return self.get_vlan_id(loadbalancer.vip.subnet_id)
+
+
+class GetMemberSubnetVLANID(GetSubnetVLANIDParent, BaseNetworkTask):
+
+    default_provides = a10constants.VLAN_ID
+
+    def execute(self, member):
+        return self.get_vlan_id(member.subnet_id)
