@@ -15,25 +15,24 @@
 import json
 import logging
 
+from oslo_config import cfg
+
 from a10_octavia.common.data_models import Certificate
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
 def get_cert_data(barbican_client, listener):
     cert_data = Certificate()
     cert_ref = listener.tls_certificate_id
-    try:
-        cert_container = barbican_client.containers.get(container_ref=cert_ref)
-        cert_data = Certificate(cert_filename=cert_container.certificate.name,
-                                key_filename=cert_container.private_key.name,
-                                cert_content=cert_container.certificate.payload,
-                                key_content=cert_container.private_key.payload,
-                                key_pass=cert_container.private_key_passphrase,
-                                template_name=listener.id)
-    except Exception as e:
-        LOG.exception("Failed to fetch cert containers: %s", str(e))
-        raise
+    cert_container = barbican_client.containers.get(container_ref=cert_ref)
+    cert_data = Certificate(cert_filename=cert_container.certificate.name,
+                            key_filename=cert_container.private_key.name,
+                            cert_content=cert_container.certificate.payload,
+                            key_content=cert_container.private_key.payload,
+                            key_pass=cert_container.private_key_passphrase,
+                            template_name=listener.id)
     return cert_data
 
 
